/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.wpni.gb.managed.beans;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.Serializable;
import java.text.Collator;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

import javax.faces.context.FacesContext;

import org.apache.log4j.Logger;
import org.richfaces.event.UploadEvent;
import org.richfaces.model.UploadItem;

import com.wpni.gb.beans.properties.GBProperties;
import com.wpni.gb.beans.service.IGalleryService;
import com.wpni.gb.constants.IConstants;
import com.wpni.gb.exception.GalleryBuilderException;
import com.wpni.gb.managed.beans.util.MessageUtil;
import com.wpni.gb.orm.mapping.Gallery;
import com.wpni.gb.utils.LogUtil;

/**
 * JSF Managed Bean for uploading media files by browsing the Server Side 
 * directories. The class delegates Business Processing to GalleryService.
 * @author bhaskarr
 */
public class MediaUploadBean implements Serializable {

    private static transient final Logger vLog = Logger.getLogger(MediaUploadBean.class);
    private List<UploadItem> files = new ArrayList<UploadItem>();
    private transient GalleryView galleryView;
    private transient IGalleryService galleryService;
    private transient GBProperties properties;   

    public MediaUploadBean() {
    }

    public GalleryView getGalleryView() {
        return galleryView;
    }

    public void setGalleryView(GalleryView galleryView) {
        this.galleryView = galleryView;
    }

    public IGalleryService getGalleryService() {
        return galleryService;
    }

    public void setGalleryService(IGalleryService galleryService) {
        this.galleryService = galleryService;
    }

    public GBProperties getProperties() {
        return properties;
    }

    public void setProperties(GBProperties properties) {
        this.properties = properties;
    }

    public List<UploadItem> getFiles() {
        return files;
    }

    public void setFiles(List<UploadItem> files) {
        this.files = files;
    }     
    
    public void listener(UploadEvent event) {
        vLog.info("File : '" + event.getUploadItems() + "' was uploaded");
    }
    
    public String resize() {
        List<UploadItem> unordereditems = getFiles();
        vLog.info("items size: " + unordereditems.size());
        if (!unordereditems.isEmpty()) {
            List<UploadItem> items = unordereditems;
            
            try {
                File[] filesArr = new File[items.size()];
                int count = 0;
                for (UploadItem item : items) {
                    File file = item.getFile();
                    StringBuilder vInvalidBuffer = new StringBuilder();
                    String vFileName = file.getName();
                    if (vFileName.startsWith(".") || vFileName.startsWith("._")) {
                        vInvalidBuffer.append(
                                MessageFormat.format("''{0}'' \n", new Object[]{vFileName}));
                    }

                    if (vFileName.trim().contains(" ")) {
                        vInvalidBuffer.append(
                                MessageFormat.format("White Space in filename - ''{0}'' \n", new Object[]{vFileName}));
                    }

                    if (vInvalidBuffer.length() > 0) {
                        throw new GalleryBuilderException("Upload aborted due to invalid files \n" + vInvalidBuffer.toString());
                    } else {
                        DataInputStream dstream = new DataInputStream(new FileInputStream(file.getAbsolutePath()));
                        byte[] bytes = new byte[dstream.available()];
                        dstream.readFully(bytes);

                        File toResizeFile = new File(item.getFileName());
                        FileOutputStream stream = new FileOutputStream(toResizeFile);
                        stream.write(bytes);
                        stream.flush();
                        stream.close();
                        filesArr[count] = toResizeFile;
                        count++;
                    }
                }//for
                
                //Editorial wanted uploads to be sorted by name even when uploaded in random order
                Arrays.sort(filesArr, new Comparator<File>() {
                    
			    private Collator c = Collator.getInstance();
			    public int compare(File f1, File f2) {
				if (f1.equals(f2)) {
				    return 0;
				}
				if (f1.isDirectory() && f2.isFile()) {
				    return -1;
				}
				if (f1.isFile() && f2.isDirectory()) {
				    return 1;
				}
				return c.compare(f1.getName(), f2.getName());
			    }
				});
                
                Gallery vGalleryVO = getGalleryView().getGalleryVO();
                getGalleryService().uploadMedia(vGalleryVO, filesArr, galleryView.getUserBean());
                
                vGalleryVO = getGalleryService().getGalleryById(vGalleryVO);
                getGalleryView().setGalleryVO(vGalleryVO);
                getGalleryView().setPhotosList(null);
                getGalleryView().setTeasersList(null);
                getGalleryView().setReorderList(null);
                
                FacesContext vFacesContext = FacesContext.getCurrentInstance();
                vFacesContext.getExternalContext().getSessionMap().remove("galleryView");
                
                vFacesContext.getApplication().createValueBinding("#{galleryView}").setValue(FacesContext.getCurrentInstance(), getGalleryView());
                
            } catch (Exception ex) {
                vLog.error(LogUtil.prepareLogMessage(ex, ex.getMessage()));
                MessageUtil.createMessage(FacesContext.getCurrentInstance(), ex.getMessage(), ex.getCause(), IConstants.ERROR);
            } finally {
                for(UploadItem rem : items) {
                    vLog.info("Deleting Temp File: " + rem.getFile().getAbsolutePath());
                    rem.getFile().delete();
                }
                files.clear();
            }
        }
        return "reload";
    }     

}// class
